from subprocess import Popen, PIPE
from time import sleep
from threading import Timer
import os.path
import ctypes

# Complete path to MBR_ShowLicence.exe
MbrolaxPath = os.path.dirname(os.path.abspath(__file__))
# Py2Exe support:
if os.path.isfile(MbrolaxPath): MbrolaxPath = os.path.dirname(MbrolaxPath)
cmd = [os.path.join(MbrolaxPath, "MBR_ShowLicence.exe")]

# Extract functions we need from user32.dll:
EnumWindows              = ctypes.windll.user32.EnumWindows
GetWindowThreadProcessId = ctypes.windll.user32.GetWindowThreadProcessId
SetForegroundWindow      = ctypes.windll.user32.SetForegroundWindow
SetActiveWindow          = ctypes.windll.user32.SetActiveWindow
BringWindowToTop         = ctypes.windll.user32.BringWindowToTop
SetFocus                 = ctypes.windll.user32.SetFocus

def processid(handle):
    """Return the ID of process that controls window specified by handle.
    Function is taken and adapted from pywinauto package.
    """
    process_id = ctypes.c_int()
    GetWindowThreadProcessId(handle, ctypes.byref(process_id))
    return process_id.value

def enum_windows():
    """Return a list of handles of all the top level windows.
    Function is taken and adapted from pywinauto package.
    """
    windows = []
    # The callback function that will be called for each HWND
    # all we do is append the wrapped handle
    def EnumWindowProc(hwnd, lparam):
        "Called for each window - adds handles to a list"
        windows.append(hwnd)
        return True
    # define the type of the child procedure
    enum_win_proc = ctypes.WINFUNCTYPE(
        ctypes.c_int, ctypes.c_long, ctypes.c_long)
    # 'construct' the callback with our function
    proc = enum_win_proc(EnumWindowProc)
    # loop over all the children (callback called for each)
    EnumWindows(proc, 0)
    # return the collected wrapped windows
    return windows

def gethandle (procid):
    """Returns the handle of a window shown by the process specified by procid.
    Function is inspired by pywinauto's function findwindows()
    """
    hwnds = enum_windows()
    pids  = [processid(x) for x in hwnds]
    return hwnds[pids.index(procid)]

def show ():
    """This function will call 'MBR_ShowLicence.exe' which does nothing except asking OS to load 'mbrola.dll'.
    This will cause Mbrola Licence dialog to appear if 'mbrola.dll' wasn't active before in this user session.
    We have to do this by calling another process, because our would otherwise block until user clicks 'I Agree'.
    And that's most definitely unacceptable.
    For some reason the dialog mbrola uses doesn't set a focus to itself properly.
    While this is not a problem for sighted persons, for blind it is a great problem.
    Screen readers don't report the window on the screen, but it is there and it influences other work.
    So, we will focus it, or better to say, force it to come to foreground, manually.
    We of course can just ask MBR_ShowLicence.exe to load the DLL, wait for window and then kill it.
    Then it won't show at all, but would this be legal?
    This function returns a boolean which if True means that the licence dialog is being shown to the user.
    False means that dialog wasn't shown at all, hence mbrola.dll was loaded before in this user session.
    """
    # Load the DLL in the separate process:
    po = Popen(cmd, shell=0, stdout=PIPE, stdin=PIPE, stderr=PIPE, creationflags=0x8000000)
    po.stdin.close(); po.stderr.close(); po.stdout.close()
    # Check that process is up and running:
    if po.poll()!=None: return False
    # Find the mbrola's licence window handler:
    tries = 0
    while tries<1000:
        # Wait for window to appear on the screen (timeout = 20 secs):
        try: hwnd = gethandle(po.pid); break
        except: sleep(0.02); tries += 1
        # Check that process is running at all:
        if po.poll()!=None: break
    try:
        # We will be good and not do that:
        #po.kill() # Not immediately :D
        # Instead we will push the window into foreground:
        SetForegroundWindow(hwnd)
        SetActiveWindow(hwnd)
        BringWindowToTop(hwnd)
        SetFocus(hwnd)
        # Set a timer to terminate the dialog after 2 minutes:
        t = Timer(120, killItAnyway, (po,))
        t.start()
        return True
    except: pass # Timed out or process exited without showing the dialog
    # If the dialog didn't appear in 20 seconds, then a machine is extremely slow
    # Or something went extremely bad
    # Assume that and kill the process, not minding licenses or what will happen next:
    if tries==1000 and po.poll()==None:
        try: po.kill()
        except: pass
    return False

def killItAnyway (po):
    """This function will be called in a timer.
    Its purpose is to close the licence dialog regardless, after a period of time.
    If a person looses focus over licence window for some reason, it's difficult to restore it using only keyboard.
    So, we will let it on for 2 long minutes, and then dump it.
    We assume that the user is familiar with the licence anyway.
    Yes, I could check for focus and return it if a window is still on,
    but I feel that I am already too good for letting it show at all. Not to mention doing all that focusing stuff.
    I will not disrupt the user's work just to jump him around over stupid licences.
    """
    try:
        if po.poll()==None: po.kill()
    except: pass
